@@ -8,12 +8,35 @@ DAIA::Document - Information about a single document
 
 use strict;
 use base 'DAIA::Object';
-our $VERSION = '0.25';
+our $VERSION = '0.27';
 use Carp qw(croak);
 
 =head1 PROPERTIES
 
-id, href, message (repeatable), item (repeatable)
+=over
+
+=item message
+
+=item id
+
+The unique identifier of this document. Must be an URI.
+
+=item href
+
+An optional link to the document or to additional information.
+Must be an URI but should be an URL.
+
+=item message
+
+An optional list of L<DAIA::Message> objects. You can set message(s) with
+the C<message> accessor, with C<addMessage>, and with C<provideMessage>.
+
+=item item
+
+An optional list of L<DAIA::Item> objects with instances/copies/holdings 
+of this document.
+
+=back
 
 =cut
 
@@ -48,7 +71,7 @@ Jakob Voss C<< <jakob.voss@gbv.de> >>
 
 =head1 LICENSE
 
-Copyright (C) 2009 by Verbundzentrale Goettingen (VZG) and Jakob Voss
+Copyright (C) 2009-2010 by Verbundzentrale Goettingen (VZG) and Jakob Voss
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself, either Perl version 5.8.8 or, at
